@@ -53,7 +53,7 @@ def execute_func(env, need, func_string):
     :param func_string: string of the found function. Without [[ ]]
     :return: return value of executed function
     """
-    func_name, func_args, func_kwargs = _analyze_func_string(func_string)
+    func_name, func_args, func_kwargs = _analyze_func_string(func_string, need)
 
     if func_name not in env.needs_functions.keys():
         raise SphinxError('Unknown dynamic sphinx-needs function: {}. Found in need: {}'.format(func_name, need['id']))
@@ -85,29 +85,49 @@ def find_and_replace_node_content(node, env, need):
     :return: None
     """
     new_children = []
-    if not node.children:
-        if isinstance(node, nodes.Text):
-            func_match = func_pattern.findall(node)
+    if not node.children and isinstance(node, nodes.Text) or isinstance(node, nodes.reference):
+        if isinstance(node, nodes.reference):
+            try:
+                new_text = node.attributes['refuri']
+            except KeyError:
+                # If no refuri is set, we don't not need to modify anything.
+                # So stop here and return the untouched node.
+                return node
+        else:
             new_text = node
-            for func_string in func_match:
-                if not is_python3:
-                    func_string = func_string.encode('utf-8')
-                # sphinx is replacing ' and " with language specific quotation marks (up and down), which makes
-                # it impossible for the later used AST render engine to detect a python function call in the given
-                # string. Therefor a replacement is needed for the execution of the found string.
-                func_string_org = func_string[:]
-                func_string = func_string.replace('„', '"')
-                func_string = func_string.replace('“', '"')
-                func_string = func_string.replace('”', '"')
-                func_string = func_string.replace('”', '"')
+        func_match = func_pattern.findall(new_text)
+        for func_string in func_match:
+            if not is_python3:
+                func_string = func_string.encode('utf-8')
+            # sphinx is replacing ' and " with language specific quotation marks (up and down), which makes
+            # it impossible for the later used AST render engine to detect a python function call in the given
+            # string. Therefor a replacement is needed for the execution of the found string.
+            func_string_org = func_string[:]
+            func_string = func_string.replace('„', '"')
+            func_string = func_string.replace('“', '"')
+            func_string = func_string.replace('”', '"')
+            func_string = func_string.replace('”', '"')
 
-                func_string = func_string.replace('‘', '"')
-                func_string = func_string.replace('’', '"')
-                func_return = execute_func(env, need, func_string)
-                if not is_python3:
-                    new_text = new_text.replace(u'[[{}]]'.format(func_string_org.decode('utf-8')), func_return)
-                else:
-                    new_text = new_text.replace(u'[[{}]]'.format(func_string_org), func_return)
+            func_string = func_string.replace('‘', '\'')
+            func_string = func_string.replace('’', '\'')
+            func_return = execute_func(env, need, func_string)
+
+            # This should never happen, but we can not be sure.
+            if isinstance(func_return, list):
+                func_return = ", ".join(func_return)
+
+            if not is_python3:
+                new_text = new_text.replace(u'[[{}]]'.format(func_string_org.decode('utf-8')), func_return)
+            else:
+                new_text = new_text.replace(u'[[{}]]'.format(func_string_org), func_return)
+        if isinstance(node, nodes.reference):
+            node.attributes['refuri'] = new_text
+            # Call normal handling for children of reference node (will contain related Text node with link-text)
+            for child in node.children:
+                new_child = find_and_replace_node_content(child, env, need)
+                new_children.append(new_child)
+                node.children = new_children
+        else:
             node = nodes.Text(new_text, new_text)
         return node
     else:
@@ -139,7 +159,7 @@ def resolve_dynamic_values(env):
     needs = env.needs_all_needs
     for key, need in needs.items():
         for need_option in need:
-            if need_option in ['docname', 'lineno', 'target_node', 'content']:
+            if need_option in ['docname', 'lineno', 'target_node', 'content', 'content_node']:
                 # dynamic values in this data are not allowed.
                 continue
             if not isinstance(need[need_option], (list, set)):
@@ -183,7 +203,10 @@ def resolve_dynamic_values(env):
                             new_values.append(element.replace('[[{}]]'.format(func_call), str(func_return)))
                         else:
                             if isinstance(need[need_option], (list, set)):
-                                new_values += func_return
+                                if isinstance(func_return, (list, set)):
+                                    new_values += func_return
+                                else:
+                                    new_values += [func_return]
 
                 need[need_option] = new_values
 
@@ -191,6 +214,35 @@ def resolve_dynamic_values(env):
     env.needs_workflow['dynamic_values_resolved'] = True
 
 
+def check_and_get_content(content, need, env):
+    """
+    Checks if the given content is a function call.
+    If not, content is returned.
+    If it is, the functions gets executed and its returns value replaces the related part in content.
+
+    :param content: option content string
+    :param need: need
+    :param env: Sphinx environment object
+    :return: string
+    """
+
+    try:
+        content = str(content)
+    except UnicodeEncodeError:
+        content = content.encode('utf-8')
+
+    func_match = func_pattern.search(content)
+    if func_match is None:
+        return content
+
+    func_call = func_match.group(1)  # Extract function call
+    func_return = execute_func(env, need, func_call)  # Execute function call and get return value
+
+    # Replace the function_call with the calculated value
+    content = content.replace('[[{}]]'.format(func_call), func_return)
+    return content
+
+
 def _detect_and_execute(content, need, env):
     try:
         content = str(content)
@@ -206,10 +258,9 @@ def _detect_and_execute(content, need, env):
 
     return func_call, func_return
 
-
-def _analyze_func_string(func_string):
+def _analyze_func_string(func_string, need):
     """
-    Analyze given functiion string an extract:
+    Analyze given function string and extract:
 
     * function name
     * function arguments
@@ -220,7 +271,10 @@ def _analyze_func_string(func_string):
     :param func_string: string of the function
     :return: function name, arguments, keyword arguments
     """
-    func = ast.parse(func_string)
+    try:
+        func = ast.parse(func_string)
+    except SyntaxError as e:
+        raise SphinxError('Parsing function string failed for need {}: {}. {}'.format(need['id'], func_string, e))
     try:
         func_call = func.body[0].value
         func_name = func_call.func.id
@@ -243,8 +297,21 @@ def _analyze_func_string(func_string):
                 elif isinstance(element, ast.Str):
                     arg_list.append(element.s)
             func_args.append(arg_list)
+        elif isinstance(arg, ast.Attribute):
+            if arg.value.id == 'need' and need is not None:
+                func_args.append(need[arg.attr])
+            else:
+                raise FunctionParsingException('usage of need attribute not supported.')
+        elif isinstance(arg, ast.NameConstant):
+            if isinstance(arg.value, bool):
+                func_args.append(arg.value)
+            else:
+                raise FunctionParsingException(
+                    'Unsupported type found in function definition. Supported are numbers, strings, bool and list')
         else:
-            raise FunctionParsingException()
+            raise FunctionParsingException(
+                'Unsupported type found in function definition: {}. '
+                'Supported are numbers, strings, bool and list'.format(func_string))
     func_kargs = {}
     for keyword in func_call.keywords:
         kvalue = keyword.value
