@@ -16,18 +16,22 @@ from sphinxcontrib.needs.directives.needimport import Needimport, NeedimportDire
 from sphinxcontrib.needs.directives.needtable import Needtable, NeedtableDirective, process_needtables
 from sphinxcontrib.needs.directives.needlist import Needlist, NeedlistDirective, process_needlist
 from sphinxcontrib.needs.directives.needflow import Needflow, NeedflowDirective, process_needflow
+from sphinxcontrib.needs.directives.needpie import Needpie, NeedpieDirective, process_needpie
+from sphinxcontrib.needs.directives.needextract import Needextract, NeedextractDirective, process_needextract
 
 from sphinxcontrib.needs.builder import NeedsBuilder
 from sphinxcontrib.needs.directives.needfilter import Needfilter, NeedfilterDirective, process_needfilters
 from sphinxcontrib.needs.environment import install_styles_static_files, install_datatables_static_files, \
-    install_collapse_static_files
+    install_collapse_static_files, install_feather_icons
 from sphinxcontrib.needs.roles.need_incoming import Need_incoming, process_need_incoming
 from sphinxcontrib.needs.roles.need_outgoing import Need_outgoing, process_need_outgoing
 from sphinxcontrib.needs.roles.need_ref import Need_ref, process_need_ref
 from sphinxcontrib.needs.roles.need_part import NeedPart, process_need_part
 from sphinxcontrib.needs.roles.need_count import NeedCount, process_need_count
-from sphinxcontrib.needs.utils import process_dynamic_values
 from sphinxcontrib.needs.functions import register_func, needs_common_functions
+from sphinxcontrib.needs.warnings import process_warnings
+from sphinxcontrib.needs.utils import INTERNALS
+from sphinxcontrib.needs.defaults import DEFAULT_DIAGRAM_TEMPLATE, LAYOUTS, NEEDFLOW_CONFIG_DEFAULTS
 
 sphinx_version = sphinx.__version__
 if parse_version(sphinx_version) >= parse_version("1.6"):
@@ -37,79 +41,7 @@ else:
 
     logging.basicConfig()  # Only need to do this once
 
-VERSION = '0.3.14'
-
-DEFAULT_TEMPLATE_COLLAPSE = """
-.. _{{id}}:
-
-{% if hide == false -%}
-.. role:: needs_tag
-.. role:: needs_status
-.. role:: needs_type
-.. role:: needs_id
-.. role:: needs_title
-
-.. rst-class:: need
-.. rst-class:: need_{{type_name}}
-
-.. container:: need
-
-    .. container:: toggle
-
-        .. container:: header
-
-            :needs_type:`{{type_name}}`: {% if title %}:needs_title:`{{title}}`{% endif %} :needs_id:`{{id}}`
-
-{% if status and  status|upper != "NONE" and not hide_status %}        | status: :needs_status:`{{status}}`{% endif %}
-{% if tags and not hide_tags %}        | tags: :needs_tag:`{{tags|join("` :needs_tag:`")}}`{% endif %}
-        | links incoming: :need_incoming:`{{id}}`
-        | links outgoing: :need_outgoing:`{{id}}`
-
-    {{content|indent(4) }}
-
-{% endif -%}
-"""
-
-DEFAULT_TEMPLATE = """
-.. _{{id}}:
-
-{% if hide == false -%}
-.. role:: needs_tag
-.. role:: needs_status
-.. role:: needs_type
-.. role:: needs_id
-.. role:: needs_title
-
-.. rst-class:: need
-.. rst-class:: need_{{type_name}}
-
-.. container:: need
-
-    :needs_type:`{{type_name}}`: :needs_title:`{{title}}` :needs_id:`{{id}}`
-
-
-{%- if status and  status|upper != "NONE" and not hide_status %}
-        | status: :needs_status:`{{status}}`
-{%- endif %}
-{%- if tags and not hide_tags %}
-        | tags: :needs_tag:`{{tags|join("` :needs_tag:`")}}`
-{%- endif %}
-        | links incoming: :need_incoming:`{{id}}`
-        | links outgoing: :need_outgoing:`{{id}}`
-
-    {{content|indent(4) }}
-
-{% endif -%}
-"""
-
-DEFAULT_DIAGRAM_TEMPLATE = \
-    """
-{%- if is_need -%}
-<size:12>{{type_name}}</size>\\n**{{title|wordwrap(15, wrapstring='**\\\\n**')}}**\\n<size:10>{{id}}</size>
-{%- else -%}
-<size:12>{{type_name}} (part)</size>\\n**{{content|wordwrap(15, wrapstring='**\\\\n**')}}**\\n<size:10>{{id_parent}}.**{{id}}**</size>
-{%- endif -%}
-"""
+VERSION = '0.5.4'
 
 
 def setup(app):
@@ -123,9 +55,6 @@ def setup(app):
                           # Kept for backwards compatibility
                           dict(directive="need", title="Need", prefix="N_", color="#9856a5", style="node")],
                          'html')
-    app.add_config_value('needs_template', DEFAULT_TEMPLATE, 'html')
-    app.add_config_value('needs_template_collapse', DEFAULT_TEMPLATE_COLLAPSE, 'html')
-
     app.add_config_value('needs_include_needs', True, 'html')
     app.add_config_value('needs_need_name', "Need", 'html')
     app.add_config_value('needs_spec_name', "Specification", 'html')
@@ -142,8 +71,6 @@ def setup(app):
     app.add_config_value('needs_table_columns', "ID;TITLE;STATUS;TYPE;OUTGOING;TAGS", 'html')
     app.add_config_value('needs_table_style', "DATATABLES", 'html')
 
-    app.add_config_value('needs_collapse_details', True, 'html')
-
     app.add_config_value('needs_role_need_template', u"{title} ({id})", 'html')
     app.add_config_value('needs_role_need_max_title_length', 30, 'html')
 
@@ -156,16 +83,15 @@ def setup(app):
                          DEFAULT_DIAGRAM_TEMPLATE,
                          'html')
 
-    # app.add_config_value('needs_functions', None, 'html')
+    app.add_config_value('needs_functions', [], 'html')
     app.add_config_value('needs_global_options', {}, 'html')
-    app.add_config_value('needs_hide_options', [], 'html')
 
     # If given, only the defined status are allowed.
     # Values needed for each status:
     # * name
     # * description
     # Example: [{"name": "open", "description": "open status"}, {...}, {...}]
-    app.add_config_value('needs_statuses', False, 'html')
+    app.add_config_value('needs_statuses', [], 'html')
 
     # If given, only the defined tags are allowed.
     # Values needed for each tag:
@@ -192,6 +118,15 @@ def setup(app):
     app.add_config_value('needs_flow_show_links', False, 'html')
     app.add_config_value('needs_flow_link_types', ["links"], 'html')
 
+    app.add_config_value('needs_warnings', {}, 'html')
+    app.add_config_value('needs_layouts', {}, 'html')
+    app.add_config_value('needs_default_layout', 'clean', 'html')
+    app.add_config_value('needs_default_style', None, 'html')
+
+    app.add_config_value('needs_flow_configs', {}, 'html')
+
+    app.add_config_value('needs_template_folder', 'needs_templates/', 'html')
+
     # Define nodes
     app.add_node(Need, html=(html_visit, html_depart), latex=(latex_visit, latex_depart))
     app.add_node(Needfilter, )
@@ -199,6 +134,8 @@ def setup(app):
     app.add_node(Needlist)
     app.add_node(Needtable)
     app.add_node(Needflow)
+    app.add_node(Needpie)
+    app.add_node(Needextract)
     app.add_node(NeedPart, html=(visitor_dummy, visitor_dummy), latex=(visitor_dummy, visitor_dummy))
 
     ########################################################################
@@ -206,6 +143,8 @@ def setup(app):
     ########################################################################
 
     # Define directives
+    # ToDo: Directives can be registered after event 'config-inited'. No need to import conf.py by our own.
+    # ToDo: See Sphinx-Test-Reports for a working implementation.
     # As values from conf.py are not available during setup phase, we have to import and read them by our own.
     # Otherwise this "app.config.needs_types" would always return the default values only.
     try:
@@ -264,14 +203,15 @@ def setup(app):
     for type in types:
         # Register requested types of needs
         app.add_directive(type["directive"], NeedDirective)
-        app.add_directive("{0}_list".format(type["directive"]), NeedDirective)
+        # app.add_directive("{0}_list".format(type["directive"]), NeedDirective)
 
     app.add_directive('needfilter', NeedfilterDirective)
     app.add_directive('needlist', NeedlistDirective)
     app.add_directive('needtable', NeedtableDirective)
     app.add_directive('needflow', NeedflowDirective)
-
+    app.add_directive('needpie', NeedpieDirective)
     app.add_directive('needimport', NeedimportDirective)
+    app.add_directive('needextract', NeedextractDirective)
 
     ########################################################################
     # ROLES
@@ -292,7 +232,7 @@ def setup(app):
     app.add_role('need_part', XRefRole(nodeclass=NeedPart,
                                        innernodeclass=nodes.inline,
                                        warn_dangling=True))
-    # Shortcut for need_inline
+    # Shortcut for need_part
     app.add_role('np', XRefRole(nodeclass=NeedPart,
                                 innernodeclass=nodes.inline,
                                 warn_dangling=True))
@@ -307,19 +247,34 @@ def setup(app):
     # Make connections to events
     app.connect('env-purge-doc', purge_needs)
     app.connect('env-before-read-docs', prepare_env)
+    app.connect('config-inited', check_configuration)
+
+    # There is also the event doctree-read.
+    # But it looks like in this event no references are already solved, which
+    # makes trouble in our code.
+    # However, some sphinx-internal actions (like image collection) are already called during
+    # doctree-read. So manipulating the doctree may result in conflicts, as e.g. images get not
+    # registered for sphinx. So some sphinx-internal tasks/functions may be called by hand again...
+    # See also https://github.com/sphinx-doc/sphinx/issues/7054#issuecomment-578019701 for an example
     app.connect('doctree-resolved', add_sections)
     app.connect('doctree-resolved', process_need_nodes)
-    app.connect('doctree-resolved', process_dynamic_values)
+    app.connect('doctree-resolved', process_needextract)
     app.connect('doctree-resolved', process_needfilters)
     app.connect('doctree-resolved', process_needlist)
     app.connect('doctree-resolved', process_needtables)
     app.connect('doctree-resolved', process_needflow)
+    app.connect('doctree-resolved', process_needpie)
     app.connect('doctree-resolved', process_need_part)
     app.connect('doctree-resolved', process_need_ref)
     app.connect('doctree-resolved', process_need_incoming)
     app.connect('doctree-resolved', process_need_outgoing)
     app.connect('doctree-resolved', process_need_count)
+    app.connect('build-finished', process_warnings)
     app.connect('env-updated', install_datatables_static_files)
+    # app.connect('env-updated', install_feather_icons)
+
+    # Called during consistency check, which if after everything got read in.
+    # app.connect('env-check-consistency', process_warnings)
 
     # Call this after all JS files, which perform DOM manipulation, have been called.
     # Otherwise newly added dom objects can not be collapsed
@@ -328,7 +283,9 @@ def setup(app):
     # This should be called last, so that need-styles can override styles from used libraries
     app.connect('env-updated', install_styles_static_files)
 
-    return {'version': VERSION}  # identifies the version of our extension
+    return {'version': VERSION,
+            'parallel_read_safe': False,  # Must be False, otherwise IDs are not found exceptions are raised.
+            'parallel_write_safe': True}
 
 
 def visitor_dummy(*args, **kwargs):
@@ -369,7 +326,6 @@ def prepare_env(app, env, docname):
         for needs_func in needs_functions:
             register_func(env, needs_func)
 
-    app.config.needs_hide_options += ['hidden']
     app.config.needs_extra_options['hidden'] = directives.unchanged
 
     # The default link name. Must exist in all configurations. Therefore we set it here for the user.
@@ -385,13 +341,17 @@ def prepare_env(app, env, docname):
         common_links.append({
             'option': 'links',
             'outgoing': 'links outgoing',
-            'incoming': 'link incoming',
+            'incoming': 'links incoming',
             'copy': False,
             'color': '#000000'
         })
 
     app.config.needs_extra_links = common_links + app.config.needs_extra_links
 
+    app.config.needs_layouts = {**LAYOUTS, **app.config.needs_layouts}
+
+    app.config.needs_flow_configs.update(NEEDFLOW_CONFIG_DEFAULTS)
+
     if not hasattr(env, 'needs_workflow'):
         # Used to store workflow status information for already executed tasks.
         # Some tasks like backlink_creation need be be performed only once.
@@ -403,3 +363,39 @@ def prepare_env(app, env, docname):
         }
         for link_type in app.config.needs_extra_links:
             env.needs_workflow['backlink_creation_{}'.format(link_type['option'])] = False
+
+
+def check_configuration(app, config):
+    """
+    Checks the configuration for invalid options.
+
+    E.g. defined need-option, which is already defined internally
+
+    :param app:
+    :param config:
+    :return:
+    """
+    extra_options = config['needs_extra_options']
+    link_types = [x['option'] for x in config['needs_extra_links']]
+
+    # Check for usage of internal names
+    for internal in INTERNALS:
+        if internal in extra_options.keys():
+            raise NeedsConfigException('Extra option "{}" already used internally. '
+                                       ' Please use another name.'.format(internal))
+        if internal in link_types:
+            raise NeedsConfigException('Link type name "{}" already used internally. '
+                                       ' Please use another name.'.format(internal))
+
+    # Check if option and link are using the same name
+    for link in link_types:
+        if link in extra_options.keys():
+            raise NeedsConfigException('Same name for link type and extra option: {}.'
+                                       ' This is not allowed.'.format(link))
+        if link + '_back' in extra_options.keys():
+            raise NeedsConfigException('Same name for automatically created link type and extra option: {}.'
+                                       ' This is not allowed.'.format(link + '_back'))
+
+
+class NeedsConfigException(SphinxError):
+    pass
